#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif


#ifdef _WIN32
#include "windows.h"
#endif

#include "stdlib.h"
#include "string.h"
#include "math.h"

#include "debug.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "auxiliar.h"
#include "2DCMC.h"
#include "Symbol.h"
#include "GLTile.h"
#include "keyboardstate.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "KObject.h"

// #include "GMap.h"

#include "KObjectCreator.h"

KObject::KObject(int x, int y, int sfx_volume)
{
    m_class = kobject_symbol;
    m_sfx_volume = sfx_volume;
    m_x = float(x);
    m_y = float(y);
    m_last_tile_used = 0;
    m_state = -1;
    m_state_cycle = 0;

    m_n_states = 0;
    m_state_animations = 0;
    m_current_animation = 0;

    m_keep_angle = false;

} /* KObject::KObject */


KObject::~KObject()
{
	int i;
	for(i=0;i<m_n_states;i++) {
		delete m_state_animations[i];
		m_state_animations[i] = 0;
	}
	if (m_state_animations!=0) delete []m_state_animations;
} /* KObject::~KObject */


bool KObject::cycle(List<class VirtualController> *vcl, Knightmare *game, GLTManager *GLTM, SFXManager *SFXM)
{
	if (m_state>=0 && m_state<m_n_states) {
//		output_debug_message("KObject: animations pointer: %p\n",m_state_animations);
		Animation *a = m_state_animations[m_state];
//		output_debug_message("KObject: cycle animation %p\n",a);
		if (a!=0 && a!=m_current_animation) {
			a->smoothTransition(m_current_animation);
			m_current_animation = a;
//			output_debug_message("KObject: started a new animation for state %i.\n",m_state);
		}

	}
	if (m_current_animation!=0) m_last_tile_used = m_current_animation->cycle();

    return true;
} /* KObject::cycle */


void KObject::draw(float angle,GLTManager *GLTM)
{
	draw(angle,0,0,GLTM);
} /* KObject::draw */


void KObject::draw(float angle,int x_offset,int y_offset,GLTManager *GLTM)
{
	if (m_last_tile_used!=0) {
		if (m_keep_angle && angle!=0) {
			glPushMatrix();
			glTranslatef(m_x+x_offset,m_y+y_offset,0);
			glRotatef(angle,1,0,0);
			m_last_tile_used->draw(0,0,0,0,1);
			glPopMatrix();
		} else {
			m_last_tile_used->draw(m_x+x_offset,m_y+y_offset,0,0,1);
		}
	}
} /* KObject::draw */


bool KObject::is_a(Symbol *c)
{
    return c->cmp(kobject_symbol);
} /* KObject::is_a */


bool KObject::is_a(char *c)
{
    bool retval;
    Symbol *s = new Symbol(c);

    retval = is_a(s);

    delete s;

    return retval;
} /* KObject::is_a */


bool KObject::collision(KObject *o)
{
    if (m_last_tile_used == 0)
        return false;
    return o->collision(m_last_tile_used, (int)m_x, (int)m_y);
} /* KObject::collision */


bool KObject::collision(int x_offs,int y_offs,KObject *o)
{
    if (m_last_tile_used == 0)
        return false;
    return o->collision(m_last_tile_used, (int)m_x+x_offs, (int)m_y+y_offs);
} /* KObject::collision */


bool KObject::collision(GLTile *t, int x, int y)
{
    if (m_last_tile_used == 0)
        return false;
    return m_last_tile_used->get_cmc()->collision_no_rotation(m_x, m_y, t->get_cmc(), float(x), float(y));
} /* KObject::collision */


bool KObject::collision(C2DCMC *c, int x, int y)
{
    if (m_last_tile_used == 0)
        return false;
    return m_last_tile_used->get_cmc()->collision_no_rotation(m_x, m_y, c, float(x), float(y));
} /* KObject::collision */



bool KObject::collision(int xoffs, int yoffs, GLTile *t, int x, int y)
{
    if (m_last_tile_used == 0)
        return false;
    return m_last_tile_used->get_cmc()->collision_no_rotation(m_x + xoffs, m_y + yoffs, t->get_cmc(), float(x), float(y));
} /* KObject::collision */


/*
 * Calculates distance between this KObject and GO_character
 * Distance will be between 1 and 255 and will be used in Mix_SetDistance
 */
int KObject::get_distance(KObject *c)
{
    int dist = 1;

    if (c != 0) {
        dist = abs(int(m_x - c->get_x()));
        // be sneaky and take some minimum and maximum distance/'sfx volume'
        // into account
        if (dist > 150) {
            dist = 150;
        } else if (dist < 25) {
            dist = 1;
        }
    }
    return dist;
}

/*
 * Calculates the 'angle' of the current KObject, based on its distance from the player
 */
int KObject::get_angle(KObject *c)
{

    float x = c->get_x() - m_x;
    float y = c->get_y() - m_y;
    int angle;

    angle = (int)(atan2(y, x) * 180 / 3.149265);
    angle = 270 - angle;

    if (angle > 360) {
        angle = 360;
    }
    if (angle < 0) {
        angle = 0;
    }
    return angle;
}
